@@ -413,9 +413,6 @@ void uwsgi_reload(char **argv) {
 #endif
 	}
 
-#ifdef UWSGI_AS_SHARED_LIBRARY
-	return;
-#else
 	uwsgi_log("running %s\n", uwsgi.binary_path);
 	uwsgi_flush_logs();
 	argv[0] = uwsgi.binary_path;
@@ -436,7 +433,6 @@ void uwsgi_reload(char **argv) {
 	uwsgi_error("execvp()");
 	// never here
 	exit(1);
-#endif
 
 }
 
@@ -957,6 +953,9 @@ struct uwsgi_stats *uwsgi_master_generate_stats() {
 			uc = uc->next;
 		}
 
+		if (uwsgi_stats_list_close(us))
+		goto end;
+
 		if (uwsgi_stats_comma(us))
 		goto end;
 	}
