@@ -290,7 +290,7 @@ void uwsgi_python_reset_random_seed() {
 
 void uwsgi_python_atexit() {
 
-	if (uwsgi.mywid == -1) goto realstuff;
+	if (uwsgi.mywid == 0) goto realstuff;
 
 	// if hijacked do not run atexit hooks
 	if (uwsgi.workers[uwsgi.mywid].hijacked)
