@@ -597,6 +597,9 @@ struct uwsgi_legion_node {
 	int quorum;
 	int changed;
 
+	// set to 1 first time when quorum is reached
+	int joined;
+
 	uint64_t checksum;
 
 	char *scroll;
@@ -634,6 +637,7 @@ struct uwsgi_legion_node {
 	struct uwsgi_string_list *unlord_hooks;
 	struct uwsgi_string_list *setup_hooks;
 	struct uwsgi_string_list *death_hooks;
+	struct uwsgi_string_list *join_hooks;
 	struct uwsgi_legion *next;
 	};
 
@@ -832,9 +836,9 @@ struct uwsgi_cache {
 	struct uwsgi_loop *next;
 	};
 
-	struct wsgi_request;
+struct wsgi_request;
 
-	struct uwsgi_socket {
+struct uwsgi_socket {
 	int fd;
 	char *name;
 	int name_len;
@@ -904,11 +908,11 @@ struct uwsgi_cache {
 	int lazy;
 	int shared;
 	int from_shared;
-	};
+};
 
-	struct uwsgi_server;
+struct uwsgi_server;
 
-	struct uwsgi_plugin {
+struct uwsgi_plugin {
 
 	const char *name;
 	const char *alias;
@@ -949,7 +953,7 @@ struct uwsgi_cache {
 
 	int (*spooler) (char *, char *, uint16_t, char *, size_t);
 
-	 uint16_t(*rpc) (void *, uint8_t, char **, uint16_t *, char *);
+	uint16_t(*rpc) (void *, uint8_t, char **, uint16_t *, char *);
 
 	void (*jail) (int (*)(void *), char **);
 	void (*before_privileges_drop) (void);
@@ -959,13 +963,12 @@ struct uwsgi_cache {
 
 	void (*master_cleanup) (void);
 
-
 	struct uwsgi_buffer* (*backtrace)(struct wsgi_request *);
-                struct uwsgi_buffer* (*exception_class)(struct wsgi_request *);
-                struct uwsgi_buffer* (*exception_msg)(struct wsgi_request *);
-                struct uwsgi_buffer* (*exception_repr)(struct wsgi_request *);
-                void (*exception_log)(struct wsgi_request *);
-	};
+        struct uwsgi_buffer* (*exception_class)(struct wsgi_request *);
+        struct uwsgi_buffer* (*exception_msg)(struct wsgi_request *);
+        struct uwsgi_buffer* (*exception_repr)(struct wsgi_request *);
+        void (*exception_log)(struct wsgi_request *);
+};
 
 #ifdef UWSGI_PCRE
 int uwsgi_regexp_build(char *, pcre **, pcre_extra **);
@@ -3756,9 +3759,12 @@ void uwsgi_user_unlock(int);
 
 #ifdef UWSGI_SSL
 void uwsgi_opt_legion(char *, char *, void *);
+struct uwsgi_legion *uwsgi_legion_register(char *, char *, char *, char *, char *);
 void uwsgi_opt_legion_node(char *, char *, void *);
+void uwsgi_legion_register_node(struct uwsgi_legion *, char *);
 void uwsgi_opt_legion_quorum(char *, char *, void *);
 void uwsgi_opt_legion_hook(char *, char *, void *);
+void uwsgi_legion_register_hook(struct uwsgi_legion *, char *, char *);
 void uwsgi_opt_legion_scroll(char *, char *, void *);
 void uwsgi_legion_add(struct uwsgi_legion *);
 char *uwsgi_ssl_rand(size_t);
@@ -3898,8 +3904,8 @@ char *uwsgi_get_header(struct wsgi_request *, char *, uint16_t, uint16_t *);
 void uwsgi_alarm_thread_start(void);
 void uwsgi_exceptions_handler_thread_start(void);
 
-#define uwsgi_response_add_connection_close(x) uwsgi_response_add_header(x, "Connection", 10, "close", 5)
-#define uwsgi_response_add_content_type(x, y, z) uwsgi_response_add_header(x, "Content-Type", 12, y, z)
+#define uwsgi_response_add_connection_close(x) uwsgi_response_add_header(x, (char *)"Connection", 10, (char *)"close", 5)
+#define uwsgi_response_add_content_type(x, y, z) uwsgi_response_add_header(x, (char *)"Content-Type", 12, y, z)
 
 struct uwsgi_stats_pusher_instance *uwsgi_stats_pusher_add(struct uwsgi_stats_pusher *, char *);
 
